import { Component, OnInit, OnChanges, OnDestroy, Input, SimpleChanges, ViewChild, Renderer2 } from '@angular/core';
import { MarkdownService } from 'ngx-markdown';
import { KatexOptions, errorKatexNotLoaded, MarkdownComponent } from 'ngx-markdown';
import { TranslateService } from '@ngx-translate/core';
import { PagesService, EditorialPage } from 'app/shared/service/pages-service'
import { Router } from '@angular/router';


declare let katex: any; // Magic


@Component({
    selector: 'jhi-pages-markdown-viewer',
    templateUrl: './markDownViewer.component.html',
    styleUrls: ['./markDownViewer.component.scss'],
})
export class PagesMarkDownViewerComponent implements OnInit, OnChanges, OnDestroy {
    @Input() path: string;

    page: EditorialPage;
    
    // see https://github.com/jfcere/ngx-markdown/issues/125
    private listenObj: any;
      
    @ViewChild('pageMarkDown', {static: false})
      private pageMarkDown: MarkdownComponent | null;
    
    public katexOptions: KatexOptions = {
        displayMode: true,
        throwOnError: false,
        errorColor: '#cc0000',
    };


    constructor(private translateService: TranslateService,  private pagesService: PagesService, 
          private markdownService: MarkdownService, private renderer: Renderer2, private router: Router) {
        this.path = '';
        this.page = {
            path: 'page content loading',
            content: '',
        }
        this.pageMarkDown = null;
        
        this.translateService.onLangChange.subscribe(() => {
            this.reloadContent();
        });
        
    }

    ngOnInit(): void {
        this.markdownService.renderKatex = (html: string, options?: KatexOptions) => this.renderKatex(html, options);
        
        this.reloadContent();
    }

    // eslint-disable-next-line
    ngOnChanges(changes: SimpleChanges): void {
        this.reloadContent();
    }
    
    
  ngOnDestroy(): void  {
    if (this.listenObj) {
      this.listenObj();
    }
  }

    private reloadContent():void {
        const lang = this.translateService.currentLang;
        if (this.path)
            this.pagesService.getPage(lang + this.path).subscribe(
                pageContent => { this.page = pageContent; },
                e => { this.page.content = "Die Seite " + this.path + " konnte nicht geladen werden."
                       console.error("Page " + this.path + " not loaded: " + e.message) }
            )
    }

    /** helper function for special latex rendering with katex */
    private renderKatex(html: string, options?: KatexOptions): string {
        if (typeof katex === 'undefined' || typeof katex.renderToString === 'undefined') {
            throw new Error(errorKatexNotLoaded);
        }
        const reDollar = /\$([^\s][^$]*?[^\s])\$/gm;
        const html2 = html.replace(reDollar, (_, tex) => this.renderSanitizedLatex(tex, options));
        const reMath = /<code class="language-math">((.|\n|\r])*?)<\/code>/gi;
        return html2.replace(reMath, (_, tex) => this.renderSanitizedLatex(tex, options));
    }

    /** wrapper to sanitize latex before rendering with latex */
    private renderSanitizedLatex(latex: string, options?: KatexOptions): string {
        return katex.renderToString(this.replaceConfusingCharKatex(latex), options);
    }

    /** these html entities must be reverted, in order to work with katex :-( */
    private replaceConfusingCharKatex(html: string): string {
        return html
            .replace(/&amp;/gm, '&')
            .replace(/&lt;/gm, '<')
            .replace(/&gt;/gm, '>')
            .replace(/&#183;/gm, '·')
            .replace(/&#10;/gm, '·')
            .replace(/( \\ )+/gm, ' \\\\ ');
    }
    
    
    public fixLinksToGoViaRouter():void {
    // because MarkdownComponent isn't 'compiled' the links don't use the angular router,
    // so I'll catch the link click events here and pass them to the router...
    if (this.pageMarkDown) {
      this.listenObj = this.renderer.listen(this.pageMarkDown.element.nativeElement, 'click', (e: Event) => {
        if (e.target && (e.target as any).tagName === 'A') {
          const el = (e.target as HTMLElement);
          const linkURL = el.getAttribute && el.getAttribute('href');
          if (linkURL && linkURL.startsWith('/pages')) {
            e.preventDefault();
            this.router.navigate([linkURL]);
          }
        }
      });
    }
  }

}
